@@ -2,11 +2,13 @@
 # by the SolveBio package (not exported).
 
 # Store the SolveBio config in an environment
+# The default environment uses an API key (token type: "Token").
 .solveEnv <- new.env()
-.solveEnv$current <- .solveEnv
-.solveEnv$current$api_host <- Sys.getenv('SOLVEBIO_API_HOST',
-                                         unset='https://api.solvebio.com')
-.solveEnv$current$api_key <- Sys.getenv('SOLVEBIO_API_KEY', unset='')
+.solveEnv$host <- Sys.getenv('SOLVEBIO_API_HOST',
+                             unset='https://api.solvebio.com')
+.solveEnv$token <- Sys.getenv('SOLVEBIO_API_KEY', unset='')
+.solveEnv$token_type <- 'Token'
+
 
 #' login
 #'
@@ -14,7 +16,7 @@
 #'
 #' @param api_key Your SolveBio API key
 #' @param api_host SolveBio API host (default: https://api.solvebio.com)
-#' @param envir (optional) The R environment used to store API credentials.
+#' @param env (optional) The R environment used to store API credentials.
 #'
 #' @examples \dontrun{
 #' login()
@@ -24,33 +26,59 @@
 #' \url{https://docs.solvebio.com/}
 #'
 #' @export
-login <- function(api_key, api_host, envir = solvebio:::.solveEnv$current) {
+login <- function(api_key, api_host, env = solvebio:::.solveEnv) {
     if (!missing(api_key)) {
-        assign('api_key', api_key, envir=envir)
+        assign('token', api_key, envir=env)
     }
 
-    if(nchar(envir$api_key) == 0L) {
+    if(nchar(env$token) == 0L) {
         stop("No API key found. Please set the 'SOLVEBIO_API_KEY' environment variable, or specify your key as the 'api_key' parameter of this function. Your API key can be found on the Account page of the SolveBio website: https://my.solvebio.com/settings/security")
     }
 
     if (!missing(api_host)) {
-        assign('api_host', api_host, envir=envir)
+        assign('host', api_host, envir=env)
     }
 
     # Test the login
     tryCatch({
-        res <- .request('GET', 'v1/user')
-        cat(sprintf("Logged-in to %s as %s.\n", envir$api_host, res$email))
-        return(invisible(res))
+        user <- User.retrieve(env=env)
+        cat(sprintf("Logged-in to %s as %s.\n", env$host, user$email))
+        return(invisible(user))
     }, error = function(e) {
         cat(sprintf("Login failed: %s\n", e$message))
     })
 }
 
-.request = function(method, path, query, body, ...) {
-    'Perform an HTTP request to the server.'
-    env <- .solveEnv$current
 
+#' createEnv
+#'
+#' Create a new SolveBio environment.
+#'
+#' @param token A SolveBio API key or OAuth2 token
+#' @param token_type SolveBio token type (default: Token)
+#' @param host (optional) The SolveBio API host (default: https://api.solvebio.com) 
+#'
+#' @examples \dontrun{
+#' env <- createEnv("MyAPIkey")
+#' User.retrieve(env = myEnv)
+#' }
+#'
+#' @references
+#' \url{https://docs.solvebio.com/}
+#'
+#' @export
+createEnv <- function(token, token_type="Token", host="https://api.solvebio.com") {
+    newEnv <- new.env()
+    newEnv$token <- token
+    newEnv$token_type <- token_type
+    newEnv$host <- host
+    return(newEnv)
+}
+
+
+# Private API request method.
+.request = function(method, path, query, body, env = solvebio:::.solveEnv, ...) {
+    'Perform an HTTP request to the server.'
     # Set defaults
     headers <- c(
                  Accept = "application/json",
@@ -58,11 +86,10 @@ login <- function(api_key, api_host, envir = solvebio:::.solveEnv$current) {
                  "Accept-Encoding" = "gzip,deflate"
                  )
 
-    if (!is.null(env$api_key) & env$api_key != "") {
-        api_key <- env$api_key
+    if (!is.null(env$token) && nchar(env$token) != 0) {
         headers <- c(
                      headers, 
-                     Authorization = paste("Token", api_key)
+                     Authorization = paste(env$token_type, env$token, sep=" ")
                      )
     }
 
@@ -71,7 +98,7 @@ login <- function(api_key, api_host, envir = solvebio:::.solveEnv$current) {
         path <- substring(path, 2)
     }
     
-    uri <- httr::modify_url(env$api_host, "path" = path)
+    uri <- httr::modify_url(env$host, "path" = path)
     useragent <- sprintf('SolveBio R Client %s [%s %s]',
                          packageVersion('solvebio'),
                          R.version$version.string,
